@@ -10,8 +10,10 @@ use std::cell::Cell;
 use std::collections::HashMap;
 use wasm_bindgen::{closure::Closure, JsCast, JsValue};
 use web_sys::{
-    self, Element, EventTarget, HtmlDetailsElement, HtmlElement,
-    HtmlInputElement, HtmlTextAreaElement, Node, Text,
+    self, Element, EventTarget, HtmlButtonElement, HtmlDetailsElement,
+    HtmlElement, HtmlFieldSetElement, HtmlInputElement, HtmlLinkElement,
+    HtmlMenuItemElement, HtmlOptGroupElement, HtmlOptionElement,
+    HtmlSelectElement, HtmlStyleElement, HtmlTextAreaElement, Node, Text,
 };
 
 thread_local!(static NODE_ID_COUNTER: Cell<usize> = Cell::new(1));
@@ -194,6 +196,10 @@ impl CreatedNode {
     }
 
     /// set the element attribute
+    ///
+    /// Note: this is called in a loop, so setting the attributes, and style will not be on
+    /// the same call, but on a subsequent call to each other. Using the if-else-if here for
+    /// attributes, style, function_call.
     #[track_caller]
     pub fn set_element_attribute<DSP, MSG>(
         program: &DSP,
@@ -237,55 +243,38 @@ impl CreatedNode {
                     });
             } else {
                 match *attr.name() {
-                    // we explicitly call the `set_value` function in the html element
                     "value" => {
-                        if let Some(input) =
-                            element.dyn_ref::<HtmlInputElement>()
-                        {
-                            input.set_value(&merged_plain_values);
-                        } else if let Some(textarea) =
-                            element.dyn_ref::<HtmlTextAreaElement>()
-                        {
-                            textarea.set_value(&merged_plain_values);
-                        }
+                        Self::set_value(element, &merged_plain_values);
                     }
-                    // explicitly call set_open for details
                     "open" => {
-                        log::trace!("attribute open in sauron: {:?}", element);
-                        if let Some(details) =
-                            element.dyn_ref::<HtmlDetailsElement>()
-                        {
-                            log::trace!("in sauron details open..");
-                            let is_open: bool = plain_values
-                                .first()
-                                .map(|v| {
-                                    v.get_simple()
-                                        .map(|v| v.as_bool())
-                                        .flatten()
-                                })
-                                .flatten()
-                                .unwrap_or(false);
-                            log::trace!("is_open: {}", is_open);
-                            details.set_open(is_open);
-                        }
+                        let is_open: bool = plain_values
+                            .first()
+                            .map(|v| {
+                                v.get_simple().map(|v| v.as_bool()).flatten()
+                            })
+                            .flatten()
+                            .unwrap_or(false);
+                        Self::set_open(element, is_open);
                     }
-                    // we explicitly call `set_checked` function on the html element
                     "checked" => {
-                        if let Some(input) =
-                            element.dyn_ref::<HtmlInputElement>()
-                        {
-                            let checked: bool = plain_values
-                                .first()
-                                .map(|av| {
-                                    av.get_simple()
-                                        .map(|v| v.as_bool())
-                                        .flatten()
-                                })
-                                .flatten()
-                                .unwrap_or(false);
-
-                            input.set_checked(checked);
-                        }
+                        let is_checked: bool = plain_values
+                            .first()
+                            .map(|av| {
+                                av.get_simple().map(|v| v.as_bool()).flatten()
+                            })
+                            .flatten()
+                            .unwrap_or(false);
+                        Self::set_checked(element, is_checked)
+                    }
+                    "disabled" => {
+                        let is_disabled: bool = plain_values
+                            .first()
+                            .map(|av| {
+                                av.get_simple().map(|v| v.as_bool()).flatten()
+                            })
+                            .flatten()
+                            .unwrap_or(false);
+                        Self::set_disabled(element, is_disabled);
                     }
                     _ => {
                         element
@@ -392,6 +381,107 @@ impl CreatedNode {
         html_element.focus().expect("must focus")
     }
 
+    /// explicitly call `set_checked` function on the html element
+    /// since setting the attribute to false will not unchecked it.
+    ///
+    /// There are only 2 elements where set_checked is applicable:
+    /// - input
+    /// - menuitem
+    fn set_checked(element: &Element, is_checked: bool) {
+        if let Some(input) = element.dyn_ref::<HtmlInputElement>() {
+            input.set_checked(is_checked);
+        } else if let Some(menu_item) = element.dyn_ref::<HtmlMenuItemElement>()
+        {
+            menu_item.set_checked(is_checked);
+        }
+    }
+
+    /// explicitly call set_open for details
+    /// since setting the attribute `open` to false will not close it.
+    ///
+    /// TODO: HtmlDialogElement ( but it is not supported on firefox and in safarit, only works on chrome)
+    ///
+    /// Applies to:
+    ///  - dialog
+    ///  - details
+    fn set_open(element: &Element, is_open: bool) {
+        if let Some(details) = element.dyn_ref::<HtmlDetailsElement>() {
+            details.set_open(is_open);
+        }
+    }
+
+    /// explicitly call on `set_disabled`
+    /// since setting the attribute `disabled` false will not enable it.
+    ///
+    /// These are 10 elements that we can call `set_disabled` function to.
+    /// - input
+    /// - button
+    /// - textarea
+    /// - style
+    /// - link
+    /// - select
+    /// - option
+    /// - optgroup
+    /// - fieldset
+    /// - menuitem
+    fn set_disabled(element: &Element, is_disabled: bool) {
+        if let Some(input) = element.dyn_ref::<HtmlInputElement>() {
+            input.set_disabled(is_disabled);
+        } else if let Some(btn) = element.dyn_ref::<HtmlButtonElement>() {
+            btn.set_disabled(is_disabled);
+        } else if let Some(text_area) = element.dyn_ref::<HtmlTextAreaElement>()
+        {
+            text_area.set_disabled(is_disabled);
+        } else if let Some(style_elem) = element.dyn_ref::<HtmlStyleElement>() {
+            style_elem.set_disabled(is_disabled);
+        } else if let Some(link_elem) = element.dyn_ref::<HtmlLinkElement>() {
+            link_elem.set_disabled(is_disabled);
+        } else if let Some(select) = element.dyn_ref::<HtmlSelectElement>() {
+            select.set_disabled(is_disabled);
+        } else if let Some(option) = element.dyn_ref::<HtmlOptionElement>() {
+            option.set_disabled(is_disabled);
+        } else if let Some(opt_group) = element.dyn_ref::<HtmlOptGroupElement>()
+        {
+            opt_group.set_disabled(is_disabled);
+        } else if let Some(field_set) = element.dyn_ref::<HtmlFieldSetElement>()
+        {
+            field_set.set_disabled(is_disabled);
+        } else if let Some(menu_item) = element.dyn_ref::<HtmlMenuItemElement>()
+        {
+            menu_item.set_disabled(is_disabled);
+        }
+    }
+
+    /// we explicitly call the `set_value` function in the html element
+    ///
+    /// Note: for most of other elements, setting the attribute value will do just fine.
+    //
+    // TODO:
+    //    web_sys::Attr::set_value
+    //    web_sys::AudioParam::set_value
+    //    web_sys::DomTokenList::set_value
+    //    web_sys::HtmlButtonElement::set_value
+    //    web_sys::HtmlDataElement::set_value
+    //    web_sys::HtmlLiElement::set_value
+    //    web_sys::HtmlMeterElement::set_value
+    //    web_sys::HtmlOptionElement::set_value
+    //    web_sys::HtmlOutputElement::set_value
+    //    web_sys::HtmlParamElement::set_value
+    //    web_sys::HtmlProgressElement::set_value
+    //    web_sys::HtmlSelectElement::set_value
+    //    web_sys::RadioNodeList::set_value
+    //    web_sys::SvgAngle::set_value
+    //    web_sys::SvgLength::set_value
+    //    web_sys::SvgNumber::set_value
+    fn set_value(element: &Element, value: &str) {
+        if let Some(input) = element.dyn_ref::<HtmlInputElement>() {
+            input.set_value(value);
+        } else if let Some(textarea) = element.dyn_ref::<HtmlTextAreaElement>()
+        {
+            textarea.set_value(value);
+        }
+    }
+
     /// remove element attribute,
     /// takes care of special case such as checked
     pub fn remove_element_attribute<MSG>(
@@ -403,15 +493,17 @@ impl CreatedNode {
         element.remove_attribute(attr.name())?;
 
         match *attr.name() {
-            "checked" => {
-                if let Some(input) = element.dyn_ref::<HtmlInputElement>() {
-                    input.set_checked(false);
-                }
+            "value" => {
+                Self::set_value(element, "");
             }
             "open" => {
-                if let Some(details) = element.dyn_ref::<HtmlDetailsElement>() {
-                    details.set_open(false);
-                }
+                Self::set_open(element, false);
+            }
+            "checked" => {
+                Self::set_checked(element, false);
+            }
+            "disabled" => {
+                Self::set_disabled(element, false);
             }
             _ => (),
         }
