@@ -21,9 +21,6 @@ cfg_if! {if #[cfg(feature = "with-dom")] {
     pub use js_sys;
 }}
 
-// reexport serde_json
-pub use serde_json;
-
 cfg_if! {if #[cfg(not(feature = "with-dom"))] {
     /// When event is not needed, such as just rendering the dom
     /// tree in server side application
@@ -34,13 +31,14 @@ cfg_if! {if #[cfg(not(feature = "with-dom"))] {
 pub mod html;
 #[macro_use]
 pub mod svg;
-#[macro_use]
-pub mod jss;
 mod render;
 pub mod vdom;
 
 pub use render::Render;
 
+#[doc(hidden)]
+pub use jss;
+#[doc(hidden)]
 pub use mt_dom;
 
 /// Prelude simplifies the imports from sauron
@@ -59,7 +57,6 @@ pub mod prelude {
         *,
     };
     pub use render::Render;
-    pub use serde_json;
     pub use vdom::map_msg::{AttributeMapMsg, ElementMapMsg, NodeMapMsg};
     #[cfg(feature = "with-dom")]
     pub use wasm_bindgen::prelude::*;
